@@ -32,6 +32,7 @@ import software.amazon.smithy.build.PluginContext;
 import software.amazon.smithy.codegen.core.Symbol;
 import software.amazon.smithy.codegen.core.SymbolDependency;
 import software.amazon.smithy.codegen.core.SymbolProvider;
+import software.amazon.smithy.go.codegen.GoSettings.ArtifactType;
 import software.amazon.smithy.go.codegen.integration.GoIntegration;
 import software.amazon.smithy.go.codegen.integration.ProtocolGenerator;
 import software.amazon.smithy.go.codegen.integration.RuntimeClientPlugin;
@@ -51,10 +52,12 @@ import software.amazon.smithy.model.shapes.UnionShape;
 import software.amazon.smithy.model.traits.EnumTrait;
 import software.amazon.smithy.model.transform.ModelTransformer;
 import software.amazon.smithy.utils.OptionalUtils;
+import software.amazon.smithy.utils.SmithyInternalApi;
 
 /**
  * Orchestrates Go client generation.
  */
+@SmithyInternalApi
 final class CodegenVisitor extends ShapeVisitor.Default<Void> {
 
     private static final Logger LOGGER = Logger.getLogger(CodegenVisitor.class.getName());
@@ -72,6 +75,7 @@ final class CodegenVisitor extends ShapeVisitor.Default<Void> {
     private final List<RuntimeClientPlugin> runtimePlugins = new ArrayList<>();
     private final ProtocolDocumentGenerator protocolDocumentGenerator;
     private final EventStreamGenerator eventStreamGenerator;
+    private final GoCodegenContext ctx;
 
     CodegenVisitor(PluginContext context) {
         // Load all integrations.
@@ -79,8 +83,10 @@ final class CodegenVisitor extends ShapeVisitor.Default<Void> {
         LOGGER.info("Attempting to discover GoIntegration from the classpath...");
         ServiceLoader.load(GoIntegration.class, loader)
                 .forEach(integration -> {
-                    LOGGER.info(() -> "Adding GoIntegration: " + integration.getClass().getName());
-                    integrations.add(integration);
+                    if (integration.getArtifactType().equals(ArtifactType.CLIENT)) {
+                        LOGGER.info(() -> "Adding GoIntegration: " + integration.getClass().getName());
+                        integrations.add(integration);
+                    }
                 });
         integrations.sort(Comparator.comparingInt(GoIntegration::getOrder));
 
@@ -149,6 +155,8 @@ final class CodegenVisitor extends ShapeVisitor.Default<Void> {
         protocolDocumentGenerator = new ProtocolDocumentGenerator(settings, model, writers);
 
         this.eventStreamGenerator = new EventStreamGenerator(settings, model, writers, symbolProvider, service);
+
+        this.ctx = new GoCodegenContext(model, settings, symbolProvider, fileManifest, writers, integrations);
     }
 
     private static ProtocolGenerator resolveProtocolGenerator(
@@ -204,6 +212,7 @@ final class CodegenVisitor extends ShapeVisitor.Default<Void> {
         for (GoIntegration integration : integrations) {
             integration.writeAdditionalFiles(settings, model, symbolProvider, writers::useFileWriter);
             integration.writeAdditionalFiles(settings, model, symbolProvider, writers);
+            integration.writeAdditionalFiles(ctx);
         }
 
         eventStreamGenerator.generateEventStreamInterfaces();
@@ -354,12 +363,9 @@ final class CodegenVisitor extends ShapeVisitor.Default<Void> {
             TopDownIndex topDownIndex = model.getKnowledge(TopDownIndex.class);
             Set<OperationShape> containedOperations = new TreeSet<>(topDownIndex.getContainedOperations(service));
             for (OperationShape operation : containedOperations) {
-                Symbol operationSymbol = symbolProvider.toSymbol(operation);
-
-                writers.useShapeWriter(
-                        operation, operationWriter -> new OperationGenerator(settings, model, symbolProvider,
-                                operationWriter, service, operation, operationSymbol, applicationProtocol,
-                                protocolGenerator, runtimePlugins).run());
+                writers.useShapeWriter(operation, operationWriter ->
+                        new OperationGenerator(ctx, operationWriter, operation, protocolGenerator, runtimePlugins)
+                                .run());
             }
         });
 
