@@ -16,13 +16,11 @@
 package software.amazon.smithy.go.codegen;
 
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
-import java.util.StringJoiner;
 import java.util.function.BiFunction;
 import java.util.function.Consumer;
 import java.util.function.Function;
@@ -31,9 +29,8 @@ import java.util.regex.Pattern;
 import software.amazon.smithy.codegen.core.CodegenException;
 import software.amazon.smithy.codegen.core.Symbol;
 import software.amazon.smithy.codegen.core.SymbolContainer;
-import software.amazon.smithy.codegen.core.SymbolDependency;
-import software.amazon.smithy.codegen.core.SymbolDependencyContainer;
 import software.amazon.smithy.codegen.core.SymbolReference;
+import software.amazon.smithy.codegen.core.SymbolWriter;
 import software.amazon.smithy.go.codegen.knowledge.GoUsageIndex;
 import software.amazon.smithy.model.Model;
 import software.amazon.smithy.model.loader.Prelude;
@@ -47,6 +44,7 @@ import software.amazon.smithy.model.traits.RequiredTrait;
 import software.amazon.smithy.model.traits.StringTrait;
 import software.amazon.smithy.utils.AbstractCodeWriter;
 import software.amazon.smithy.utils.ListUtils;
+import software.amazon.smithy.utils.SmithyInternalApi;
 import software.amazon.smithy.utils.StringUtils;
 
 /**
@@ -56,15 +54,15 @@ import software.amazon.smithy.utils.StringUtils;
  *
  * <p>Use the {@code $P} formatter to refer to {@link Symbol}s using pointers where appropriate.
  */
-public final class GoWriter extends AbstractCodeWriter<GoWriter> {
+@SmithyInternalApi
+public final class GoWriter extends SymbolWriter<GoWriter, ImportDeclarations> {
 
     private static final Logger LOGGER = Logger.getLogger(GoWriter.class.getName());
     private static final int DEFAULT_DOC_WRAP_LENGTH = 80;
     private static final Pattern ARGUMENT_NAME_PATTERN = Pattern.compile("\\$([a-z][a-zA-Z_0-9]+)(:\\w)?");
     private final String fullPackageName;
-    private final ImportDeclarations imports = new ImportDeclarations();
-    private final List<SymbolDependency> dependencies = new ArrayList<>();
     private final boolean innerWriter;
+    private final List<String> buildTags = new ArrayList<>();
 
     private int docWrapLength = DEFAULT_DOC_WRAP_LENGTH;
     private AbstractCodeWriter<GoWriter> packageDocs;
@@ -75,12 +73,16 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @param fullPackageName package and filename to be written to.
      */
     public GoWriter(String fullPackageName) {
+        super(new ImportDeclarations(fullPackageName));
+
         this.fullPackageName = fullPackageName;
         this.innerWriter = false;
         init();
     }
 
     private GoWriter(String fullPackageName, boolean innerWriter) {
+        super(new ImportDeclarations(fullPackageName));
+
         this.fullPackageName = fullPackageName;
         this.innerWriter = innerWriter;
         init();
@@ -93,6 +95,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
         putFormatter('T', new GoSymbolFormatter());
         putFormatter('P', new PointableGoSymbolFormatter());
         putFormatter('W', new GoWritableInjector());
+        putFormatter('D', new GoDependencyFormatter());
 
         if (!innerWriter) {
             packageDocs = new GoWriter(this.fullPackageName, true);
@@ -158,15 +161,21 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @return writer for formatted docs.
      */
     public static Writable autoDocTemplate(String contents) {
-        return GoWriter.ChainWritable.of(
-                Arrays.stream(contents.split("\n\n"))
-                        .map(it -> docParagraphWriter(it.replace("\n", " ")))
-                        .toList()
-        ).compose(false);
+        var paragraphs = contents.split("\n\n");
+        var chain = new GoWriter.ChainWritable();
+        for (int i = 0; i < paragraphs.length; ++i) {
+            chain.add(docParagraphWriter(paragraphs[i], i < paragraphs.length - 1));
+        }
+        return chain.compose(false);
     }
 
-    private static GoWriter.Writable docParagraphWriter(String paragraph) {
-        return writer -> writer.writeDocs(paragraph).writeDocs("");
+    private static GoWriter.Writable docParagraphWriter(String paragraph, boolean writeNewline) {
+        return writer -> {
+            writer.writeDocs(paragraph);
+            if (writeNewline) {
+                writer.writeDocs("");
+            }
+        };
     }
 
     @SafeVarargs
@@ -600,46 +609,12 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @return Returns the writer.
      */
     public GoWriter addUseImports(GoDependency goDependency) {
-        dependencies.addAll(goDependency.getDependencies());
+        addDependency(goDependency);
         return addImport(goDependency.getImportPath(), goDependency.getAlias());
     }
 
-    /**
-     * Imports a symbol if necessary using a package alias and list of context options.
-     *
-     * @param symbol       Symbol to optionally import.
-     * @param packageAlias The alias to refer to the symbol's package by.
-     * @param options      The list of context options (e.g., is it a USE or DECLARE symbol).
-     * @return Returns the writer.
-     */
-    public GoWriter addImport(Symbol symbol, String packageAlias, SymbolReference.ContextOption... options) {
-        LOGGER.finest(() -> {
-            StringJoiner stackTrace = new StringJoiner("\n");
-            for (StackTraceElement element : Thread.currentThread().getStackTrace()) {
-                stackTrace.add(element.toString());
-            }
-            return String.format(
-                    "Adding Go import %s as `%s` (%s); Stack trace: %s",
-                    symbol.getNamespace(), packageAlias, Arrays.toString(options), stackTrace);
-        });
-
-        // Always add dependencies.
-        dependencies.addAll(symbol.getDependencies());
-
-        if (isExternalNamespace(symbol.getNamespace())) {
-            addImport(symbol.getNamespace(), packageAlias);
-        }
-
-        // Just because the direct symbol wasn't imported doesn't mean that the
-        // symbols it needs to be declared don't need to be imported.
-        addImportReferences(symbol, options);
-
-        return this;
-    }
-
-    private GoWriter addImports(GoWriter other) {
-        this.imports.addImports(other.imports);
-        return this;
+    private void addImports(GoWriter other) {
+        this.getImportContainer().addImports(other.getImportContainer());
     }
 
     private boolean isExternalNamespace(String namespace) {
@@ -665,31 +640,12 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @return Returns the writer.
      */
     public GoWriter addImport(String packageName, String as) {
-        imports.addImport(packageName, as);
+        getImportContainer().addImport(packageName, as);
         return this;
     }
 
-    /**
-     * Adds one or more dependencies to the generated code.
-     *
-     * <p>The dependencies of all writers created by the {@link GoDelegator}
-     * are merged together to eventually generate a go.mod file.
-     *
-     * @param dependencies Go dependency to add.
-     * @return Returns the writer.
-     */
-    public GoWriter addDependency(SymbolDependencyContainer dependencies) {
-        this.dependencies.addAll(dependencies.getDependencies());
-        return this;
-    }
-
-    private GoWriter addDependencies(GoWriter other) {
-        this.dependencies.addAll(other.getDependencies());
-        return this;
-    }
-
-    Collection<SymbolDependency> getDependencies() {
-        return dependencies;
+    private void addDependencies(GoWriter other) {
+        addDependency(other);
     }
 
     /**
@@ -773,7 +729,8 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @param shape Shape to write the documentation of.
      * @return Returns true if docs were written.
      */
-    boolean writeShapeDocs(Shape shape) {
+    @SmithyInternalApi
+    public boolean writeShapeDocs(Shape shape) {
         return shape.getTrait(DocumentationTrait.class)
                 .map(DocumentationTrait::getValue)
                 .map(docs -> {
@@ -788,7 +745,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @param shape Shape to write the documentation of.
      * @return Returns true if docs were written.
      */
-    boolean writePackageShapeDocs(Shape shape) {
+    public boolean writePackageShapeDocs(Shape shape) {
         return shape.getTrait(DocumentationTrait.class)
                 .map(DocumentationTrait::getValue)
                 .map(docs -> {
@@ -804,7 +761,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
      * @param member Shape to write the documentation of.
      * @return Returns true if docs were written.
      */
-    boolean writeMemberDocs(Model model, MemberShape member) {
+    public boolean writeMemberDocs(Model model, MemberShape member) {
         boolean hasDocs;
 
         hasDocs = member.getMemberTrait(model, DocumentationTrait.class)
@@ -875,6 +832,11 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
         write("$W", w);
     }
 
+    public GoWriter addBuildTag(String tag) {
+        buildTags.add(tag);
+        return this;
+    }
+
     @Override
     public String toString() {
         String contents = super.toString();
@@ -883,6 +845,9 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
             return contents;
         }
 
+        var tags = buildTags.isEmpty()
+                ? ""
+                : "//go:build " + String.join(",", buildTags) + "\n";
 
         String[] packageParts = fullPackageName.split("/");
         String header = String.format("// Code generated by smithy-go-codegen DO NOT EDIT.%n%n");
@@ -904,7 +869,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
         String packageDocs = this.packageDocs.toString();
         String packageStatement = String.format("package %s%n%n", packageName);
 
-        String importString = imports.toString();
+        String importString = getImportContainer().toString();
         String strippedContents = StringUtils.stripStart(contents, null);
         String strippedImportString = StringUtils.strip(importString, null);
 
@@ -913,7 +878,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
             return header + strippedImportString + "\n" + strippedContents;
         }
 
-        return header + packageDocs + packageStatement + importString + contents;
+        return header + packageDocs + tags + packageStatement + importString + contents;
     }
 
     /**
@@ -1007,6 +972,22 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
         }
     }
 
+    /**
+     * Implements Go symbol formatting for the {@code $D} formatter.
+     */
+    private class GoDependencyFormatter implements BiFunction<Object, String, String> {
+        @Override
+        public String apply(Object type, String indent) {
+            if (type instanceof GoDependency) {
+                addUseImports((GoDependency) type);
+            } else {
+                throw new CodegenException(
+                        "Invalid type provided to $D. Expected a GoDependency, but found `" + type + "`");
+            }
+            return "";
+        }
+    }
+
     public interface Writable extends Consumer<GoWriter> {
     }
 
@@ -1026,7 +1007,7 @@ public final class GoWriter extends AbstractCodeWriter<GoWriter> {
             return chain;
         }
 
-        public static ChainWritable of(List<GoWriter.Writable> writables) {
+        public static ChainWritable of(Collection<GoWriter.Writable> writables) {
             var chain = new ChainWritable();
             chain.writables.addAll(writables);
             return chain;
