@@ -223,4 +223,15 @@ public final class SymbolUtils {
                 || symbol.getProperty(SymbolUtils.GO_SLICE).isPresent()
                 || symbol.getProperty(SymbolUtils.GO_MAP).isPresent();
     }
+
+    public static Symbol pointerTo(Symbol symbol) {
+        return symbol.toBuilder().putProperty(POINTABLE, true).build();
+    }
+
+    public static Symbol sliceOf(Symbol symbol) {
+        return symbol.toBuilder()
+                .putProperty(GO_SLICE, true)
+                .putProperty(GO_ELEMENT_TYPE, symbol)
+                .build();
+    }
 }
